use crate::ast::node::class::{ClassDeclaration, FieldDeclaration, PropertyDeclaration};
use crate::ast::node::function::{
    FunctionDefinition, InterfaceFunctionDeclaration, InterfaceOperatorFunctionDeclaration,
    OperatorFunctionDefinition, PlatformFunctionDeclaration,
};
use crate::ast::node::interface::InterfaceDeclaration;
use crate::ast::node::module::ModuleDeclaration;

#[derive(Debug)]
pub enum ModuleLevelDeclaration {
    Module(Box<ModuleDeclaration>),
    Interface(Box<InterfaceDeclaration>),
    Class(Box<ClassDeclaration>),
    Function(Box<FunctionDefinition>),
    PlatformFunction(Box<PlatformFunctionDeclaration>),
}

#[derive(Debug)]
pub enum InterfaceLevelDeclaration {
    Module(Box<ModuleDeclaration>),
    Interface(Box<InterfaceDeclaration>),
    Class(Box<ClassDeclaration>),
    Function(Box<InterfaceFunctionDeclaration>),
    OperatorFunction(Box<InterfaceOperatorFunctionDeclaration>),
}

#[derive(Debug)]
pub enum ClassLevelDeclaration {
    Module(Box<ModuleDeclaration>),
    Interface(Box<InterfaceDeclaration>),
    Class(Box<ClassDeclaration>),
    Function(Box<FunctionDefinition>),
    OperatorFunction(Box<OperatorFunctionDefinition>),
    PlatformFunction(Box<PlatformFunctionDeclaration>),
    Property(Box<PropertyDeclaration>),
    Field(Box<FieldDeclaration>),
}
