use crate::ast::node::level::ModuleLevelDeclaration;
use crate::ast::node::names::Identifier;

#[derive(Debug)]
pub struct ModuleDeclaration {
    is_public: bool,
    identifier: Box<Identifier>,
    declarations: Vec<Box<ModuleLevelDeclaration>>,
}

impl ModuleDeclaration {
    pub fn new(
        is_public: bool,
        identifier: Box<Identifier>,
        declarations: Vec<Box<ModuleLevelDeclaration>>,
    ) -> Self {
        Self {
            is_public,
            identifier,
            declarations,
        }
    }

    pub fn is_public(&self) -> bool {
        self.is_public
    }

    pub fn identifier(&self) -> &Identifier {
        &self.identifier
    }

    pub fn identifier_mut(&mut self) -> &mut Identifier {
        &mut self.identifier
    }

    pub fn declarations(&self) -> Vec<&ModuleLevelDeclaration> {
        self.declarations.iter().map(Box::as_ref).collect()
    }

    pub fn declarations_mut(&mut self) -> Vec<&mut ModuleLevelDeclaration> {
        self.declarations.iter_mut().map(Box::as_mut).collect()
    }
}
