use crate::ast::node::named::Named;
use crate::ast::node::names::Identifier;
use std::borrow::Cow;
use std::range::Range;

#[derive(Debug)]
pub struct UseStatement {
    identifiers: Vec<Box<Identifier>>,
    last: Box<UseStatementLast>,
    file_id: usize,
    range: Range<usize>,
}

impl UseStatement {
    pub fn new(
        identifiers: Vec<Box<Identifier>>,
        last: Box<UseStatementLast>,
        file_id: usize,
        range: Range<usize>,
    ) -> Self {
        UseStatement {
            identifiers,
            last,
            file_id,
            range,
        }
    }

    pub fn base_name(&self) -> Cow<'_, str> {
        use UseStatementLast::*;
        if self.identifiers.is_empty() {
            match self.last.as_ref() {
                Identifier(_) => Cow::from(""),
                Star | Identifiers(_) => panic!(), // should never get here because of grammar
            }
        } else if self.identifiers.len() == 1 {
            self.identifiers[0].name()
        } else {
            let mut acc = String::new();
            for (i, identifier) in self.identifiers.iter().enumerate() {
                acc.push_str(&identifier.name());
                if i != self.identifiers.len() - 1 {
                    acc.push_str("::");
                }
            }
            Cow::from(acc)
        }
    }

    pub fn is_star(&self) -> bool {
        match self.last.as_ref() {
            UseStatementLast::Star => true,
            _ => false,
        }
    }

    pub fn identifiers(&self) -> Vec<&Identifier> {
        self.identifiers.iter().map(Box::as_ref).collect()
    }

    pub fn identifiers_mut(&mut self) -> Vec<&mut Identifier> {
        self.identifiers.iter_mut().map(Box::as_mut).collect()
    }

    pub fn last(&self) -> &UseStatementLast {
        &self.last
    }

    pub fn last_mut(&mut self) -> &mut UseStatementLast {
        &mut self.last
    }

    pub fn file_id(&self) -> usize {
        self.file_id
    }

    pub fn range(&self) -> Range<usize> {
        self.range
    }
}

#[derive(Debug)]
pub enum UseStatementLast {
    Identifier(Box<Identifier>),
    Identifiers(Vec<Box<Identifier>>),
    Star,
}
